<?php

namespace Lc\SovBundle\Controller\User;

use EasyCorp\Bundle\EasyAdminBundle\Dto\EntityDto;
use EasyCorp\Bundle\EasyAdminBundle\Provider\AdminContextProvider;
use Lc\SovBundle\Doctrine\EntityManager;
use Lc\SovBundle\Form\User\ChangePasswordFormType;
use Lc\SovBundle\Form\User\ProfileFormType;
use Lc\SovBundle\Model\User\User;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Security\Core\Encoder\UserPasswordEncoderInterface;
use Symfony\Component\Translation\TranslatableMessage;

class AccountAdminController extends AbstractController
{
    protected $em;

    public function __construct(EntityManager $em)
    {
        $this->em = $em;
    }

    public function profile(Request $request): Response
    {
        $user = $this->getUser();
        $form = $this->createForm(ProfileFormType::class, $user);

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $user = $form->getData();

            $this->em->update($user);
            $this->em->flush();

            $this->addFlash('success', new TranslatableMessage('form.account_profile.message.success', [], 'admin'));
        }

        return $this->render(
                '@LcSov/user/profile.html.twig',
                [
                        'form' => $form->createView(),
                ]
        );
    }

    public function changePassword(Request $request, UserPasswordEncoderInterface $passwordEncoder): Response
    {
        $user = $this->getUser();
        $form = $this->createForm(ChangePasswordFormType::class, $user);

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $user = $form->getData();

            $plainPassword = $form->get('plain_password')->getData();

            $user->setPassword($passwordEncoder->encodePassword($user, $plainPassword));

            $this->em->update($user);
            $this->em->flush();

            $this->addFlash('success', new TranslatableMessage('form.account_password.message.success', [], 'admin'));
        }

        return $this->render(
                '@LcSov/user/change_password.html.twig',
                [
                        'entity_class' => User::class,
                        'form' => $form->createView()
                ]
        );
    }

}
